import string
from _feature_engineering import Transformer
import types

def func_copy(f):
    """
    Make a copy of a function using the underlying function attributes.
    """
    return types.FunctionType(f.func_code, f.func_globals, name = f.func_name,
                              argdefs = f.func_defaults,
                              closure = f.func_closure)
def fit(self, data):
    return Transformer.fit(self, data)

def transform(self, data):
    return Transformer.transform(self, data)

def fit_transform(self, data):
    return Transformer.fit_transform(self, data)

def republish_docs(cls):
    """
    Republish the doc-strings for fit, transform, and fit_transform.
    """
    fit_copy = func_copy(fit)
    fit_copy.func_doc = Transformer.fit.im_func.func_doc
    setattr(cls, 'fit', add_docstring(
            examples = cls._fit_examples_doc)(fit_copy))

    transform_copy = func_copy(transform)
    transform_copy.func_doc = Transformer.transform.im_func.func_doc
    setattr(cls, 'transform', add_docstring(
            examples = cls._transform_examples_doc)(transform_copy))

    fit_transform_copy = func_copy(fit_transform)
    fit_transform_copy.func_doc = Transformer.fit_transform.im_func.func_doc
    setattr(cls, 'fit_transform', add_docstring(
            examples = cls._fit_transform_examples_doc)(fit_transform_copy))
    return cls

class _Formatter(string.Formatter):
    """
    Format {strings} that are withing {brackets} as described in the doctring.
    """
    def get_value(self, key, args, kwargs):
        if hasattr(key,"__mod__") and key in args:
                return args[key]
        elif key in kwargs:
                return kwargs[key]
        return '{%s}' % key

def add_docstring(**format_dict):
    """
    __example_start = '''
       Examples
       ---------
    '''
    __create = '''
       >>> import graphlab as gl

       >>> url = 'http://s3.amazonaws.com/gl-testdata/xgboost/mushroom.csv'
       >>> data = graphlab.SFrame.read_csv(url)
       >>> data['target'] = (data['target'] == 'e')

       >>> train, test = data.random_split(0.8)
       >>> model = graphlab.boosted_trees.create(train, target='label', *args, **kwargs)
    '''

    @add_docstring(create = __create, example_start = __example_start)
    def predict(x, **kwargs):
      '''
      {example_start}{create}
      '''
      return x

    """
    def add_doc_string_context(func):
        wrapper = func
        formatter = _Formatter()
        wrapper.func_doc = formatter.format(func.func_doc, **format_dict)
        return wrapper
    return add_doc_string_context

