"""
This module provides utilities for doing text processing.

Note that standard SArray utilities can be used for transforming text data
into "bag of words" format, where a document is represented as a
dictionary mapping unique words with the number of times that word occurs
in the document. See :py:func:`~graphlab.text_analytics.count_words`
for more details. Also, see :py:func:`~graphlab.SFrame.pack_columns` and
:py:func:`~graphlab.SFrame.unstack` for ways of creating SArrays
containing dictionary types.

We provide methods for learning topic models, which can be useful for modeling
large document collections. See
:py:func:`~graphlab.topic_model.create` for more, as well as the
`How-Tos <https://dato.com/learn/how-to>`_, data science `Gallery
<https://dato.com/learn/gallery>`_, and `text analysis chapter of
the User Guide
<https://dato.com/learn/userguide/text/intro.html>`_.

"""

__all__ = ['tf_idf', 'bm25', 'stopwords', 'count_words',
           'count_ngrams', 'random_split', 'parse_sparse',
           'parse_docword', 'tokenize']
def __dir__():
  return ['tf_idf', 'bm25', 'stopwords', 'count_words',
          'count_ngrams', 'random_split', 'parse_sparse',
          'parse_docword', 'tokenize']
from _util import tf_idf
from _util import bm25
from _util import stopwords
from _util import count_words
from _util import count_ngrams
from _util import random_split
from _util import parse_sparse
from _util import parse_docword
from _util import tokenize
