import urllib2 as _urllib2
import os as _os
import ConfigParser as _ConfigParser
import graphlab.version_info as _version_info
from pkg_resources import parse_version as _parse_version
import logging as _logging
import sys as _sys

__GLCREATE_CURRENT_VERSION_URL__ = "https://dato.com/files/glcreate_current_version"
__GLCREATE_CURRENT_VERSION_FEATURES_URL__ = "https://dato.com/files/glcreate_current_features"

__LOGGER__ = _logging.getLogger(__name__)

def get_newest_version(timeout=5, _url=__GLCREATE_CURRENT_VERSION_URL__):
    """
    Returns the version of GraphLab Create currently available from dato.com.
    Will raise an exception if we are unable to reach the dato.com servers.

    timeout: int
        How many seconds to wait for the remote server to respond

    url: string
        The URL to go to to check the current version.
    """
    request = _urllib2.urlopen(url=_url, timeout=timeout)
    version = request.read()
    __LOGGER__.debug("current_version read %s" % version)
    return version


def get_newest_features(timeout=2, _url=__GLCREATE_CURRENT_VERSION_FEATURES_URL__):
    """
    Returns the features of the lastest GraphLab Create currently available from dato.com.
    Will raise an exception if we are unable to reach the dato.com servers.

    timeout: int
        How many seconds to wait for the remote server to respond

    url: string
        The URL to go to to check the current version.
    """
    request = _urllib2.urlopen(url=_url, timeout=timeout)
    version = request.read()
    __LOGGER__.debug("current_feature read %s" % version)
    return version


def get_major_version(version_str):
    return '.'.join(version_str.split('.')[:2])


def perform_version_check(configfile=(_os.path.join(_os.path.expanduser("~"), ".graphlab", "config")),
                          _version_url=__GLCREATE_CURRENT_VERSION_URL__,
                          _feature_url=__GLCREATE_CURRENT_VERSION_FEATURES_URL__,
                          _outputstream=_sys.stderr):
    """
    Checks if currently running version of GraphLab Create is less than the
    version available from dato.com. Prints a message if the dato.com
    servers are reachable, and the current version is out of date. Does
    nothing otherwise.

    If the configfile contains a key "skip_version_check" in the Product
    section with non-zero value, this function does nothing.

    Also returns True if a message is printed, and returns False otherwise.
    """
    skip_version_check = False
    try:
        if (_os.path.isfile(configfile)):
            config = _ConfigParser.ConfigParser()
            config.read(configfile)
            section = 'Product'
            key = 'skip_version_check'
            skip_version_check = config.getboolean(section, key)
            __LOGGER__.debug("skip_version_check=%s" % str(skip_version_check))
    except:
        # eat all errors
        pass

    # skip version check set. Quit
    if not skip_version_check:
        try:
            latest_version = get_newest_version(timeout=1,
                                                _url=_version_url).strip()
            if _parse_version(latest_version) > _parse_version(_version_info.version):
                try:
                    _feature_url += '/' + get_major_version(_version_info.version)
                    latest_features = get_newest_features(timeout=1,
                                                          _url=_feature_url).strip()
                except:
                    latest_features = ''
                msg = ("A newer version of GraphLab Create (v%s) is available! "
                       "Your current version is v%s.\n"
                       "%s\n"  # this is the string of new features list
                       "You can use pip to upgrade the graphlab-create package. "
                       "For more information see https://dato.com/products/create/upgrade.\n") % \
                      (latest_version, _version_info.version, latest_features)
                _outputstream.write(msg)
                return True
        except:
            # eat all errors
            pass
    return False

